/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 2.7.18 -
 */

#ifndef INC_4NEURO_PARTICLESWARM_H
#define INC_4NEURO_PARTICLESWARM_H

#include "stdlib.h"
#include "time.h"

#include "../Network/NeuralNetwork.h"
#include "../DataSet/DataSet.h"
#include "../ErrorFunction/ErrorFunctions.h"


class Particle{
private:

    size_t coordinate_dim;
    double *coordinate = nullptr;
    double *velocity;

    double *optimal_coordinate = nullptr;
    double optimal_value;

    double r1;
    double r2;

    ErrorFunction* ef;

    double *domain_bounds;

    void print_coordinate();

public:

    /**
     *
     * @param f_dim
     */
    Particle(ErrorFunction* ef, double *domain_bounds);
    ~Particle( );

    /**
     *
     * @return
     */
    double* get_coordinate();

    /**
     *
     * @return
     */
    double get_optimal_value();

    /**
     *
     * @param ref_coordinate
     */
    void get_optimal_coordinate(double *ref_coordinate);

    /**
     *
     * @param w
     * @param c1
     * @param c2
     * @param glob_min_coord
     * @param penalty_coef
     */
    double change_coordinate(double w, double c1, double c2, double* glob_min_coord, double penalty_coef=0.25);
};


class ParticleSwarm {

private:

    /**
     *
     */
    Particle** particle_swarm = nullptr;

    /**
     *
     */
    double (*func)(NeuralNetwork, double*, DataSet);

    size_t func_dim;

    unsigned int n_particles;

    unsigned int iter_max;

    double c1;

    double c2;

    double w;

    double global_optimal_value;

    double *domain_bounds;

protected:
    /**
     *
     * @param coord
     * @param val
     * @return
     */
    Particle* determine_optimal_coordinate_and_value(double *coord, double &val);

    /**
     *
     * @return
     */
    double* get_centroid_coordinates();

    /**
     *
     * @param a
     * @param b
     * @param n
     * @return
     */
    double get_euclidean_distance(double* a, double* b, unsigned int n);

public:

    /**
     *
     * @param ef
     * @param f_dim
     * @param domain_bounds
     * @param c1
     * @param c2
     * @param w
     * @param n_particles
     * @param iter_max
     */
    ParticleSwarm( ErrorFunction* ef, double* domain_bounds, double c1, double c2, double w, unsigned int n_particles, unsigned int iter_max = 1 );

    /**
     *
     */
    ~ParticleSwarm( );


    /**
     *
     * @param gamma
     * @param epsilon
     * @param delta
     */
    void optimize( double gamma, double epsilon, double delta=0.7 );


};


#endif //INC_4NEURO_PARTICLESWARM_H
