/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 13.6.18 -
 */

#ifndef INC_4NEURO_CONNECTION_H
#define INC_4NEURO_CONNECTION_H

#include "../Neuron/Neuron.h"
#include "ConnectionWeight.h"
#include "../settings.h"

class Neuron;
class ConnectionWeight;

/**
 * Class representing directed connection between two neurons
 */
class Connection {
private:

    /**
     * Pointer to an object evaluating the weight function
     */
    ConnectionWeight *con = nullptr;
    /**
     * Pointer to the Neuron on the receiving end of this connection
     */
    Neuron *neuron_in = nullptr;

    /**
     * Pointer to the Neuron on the signaling end of this connection
     */
    Neuron *neuron_out = nullptr;

    /**
     *
     */
    bool delete_connection = true;


    //TODO pridat gradient pro ucely backpropagation

public:

    /**
     * Returns a new object reprresenting an edge using the same weight function as this one
     * @param n_in
     * @param n_out
     * @return
     */
    virtual Connection* get_copy(Neuron *n_in, Neuron *n_out) final;

   /**
    *
    * @param n_in
    * @param n_out
    * @param con
    * @param delete_weight
    */
    Connection(Neuron *n_in, Neuron *n_out, ConnectionWeight* con, bool delete_weight = true);

    /**
     * Destructor, deletes the 'con' object
     */
    ~Connection();

    /**
     * Takes the array of double values and alters the corresponding weights associated
     * with the 'con' object
     * @param[in] values Values to be added to the associated weights
     */
    void adjust_weights(double *values);

    /**
     * Takes the array of double values and sets the corresponding weights associated
     * with the 'con' object
     * @param[in] values Values to be set to the associated weights
     */
    void set_weights(double *values);

    /**
     * Takes the output signal of Neuron 'neuron_out', multiplies it by the result of the
     * weight function associated with this connection and adds the result to the potential
     * of the Neuron 'neuron_in'
     */
    void pass_signal();


    /**
     * Returns the pointer to the Neuron on the receiving end of this connection
     * @return
     */
    Neuron* get_neuron_in();

    /**
     * Returns the pointer to the Neuron on the signaling end of this connection
     * @return
     */
    Neuron* get_neuron_out();

};


#endif //INC_4NEURO_CONNECTION_H
