/**
 * DESCRIPTION OF THE FILE
 *
 * @author Michal Kravčenko
 * @date 13.6.18 -
 */

 //TODO pouvazovat o pridani indexu k neuronum, abychom meli urcitou kontrolu nad poradim vstupu a vystupu?
 //TODO rucni nastaveni vstupnich a vystu[nich neuronu

#ifndef INC_4NEURO_NEURALNETWORK_H
#define INC_4NEURO_NEURALNETWORK_H

#include <vector>
#include "../Neuron/Neuron.h"
#include "../settings.h"

enum NET_TYPE{GENERAL};


/**
 *
 */
class NeuralNetwork {
private:
    /**
     *
     */
    NET_TYPE network_type = GENERAL;

     /**
      *
      */
     std::vector<Neuron*> *neurons = nullptr;

     /**
      *
      */
     std::vector<Neuron*>* input_neurons = nullptr;

     /**
      *
      */
     std::vector<Neuron*>* output_neurons = nullptr;

     std::vector<double>* connection_weights = nullptr;

     /**
      *
      */
     size_t n_inputs = 0;

     /**
      *
      */
     size_t n_outputs = 0;

     /**
      *
      */
     size_t n_weights = 0;

     /**
      *
      */
     bool in_out_determined = false;

     /**
      *
      */
     bool delete_weights = true;

     /**
      *
      */
     std::vector<Neuron*>* active_eval_set = nullptr;

     /**
      *
      */
     void determine_inputs_outputs();



public:

    /**
     *
     */
    NeuralNetwork();


    /**
     *
     */
    ~NeuralNetwork();

    /**
     * If possible, returns a neural net with 'input_neuron_indices' neurons as inputs and 'output_neuron_indices' as
     * outputs, otherwise returns nullptr. The returned object shares adjustable weights with this network. All
     * neurons are coppied (new instances), edges also. Uses a breadth-first search as the underlying algorithm.
     * @param input_neuron_indices
     * @param output_neuron_indices
     * @return
     */
    NeuralNetwork* get_subnet(std::vector<size_t> &input_neuron_indices, std::vector<size_t> &output_neuron_indices);

    /**
     *
     * @param[in] input
     * @param[in,out] output
     */
    virtual void eval_single(std::vector<double> &input, std::vector<double> &output, double *custom_weights = nullptr);



    /**
     *
     * @param[in] n
     * @return
     */
    int add_neuron(Neuron* n);

    /**
     *
     * @param n1_idx
     * @param n2_idx
     * @return
     */
    size_t add_connection_simple(int n1_idx, int n2_idx);

    /**
     *
     * @param n1_idx
     * @param n2_idx
     * @param weight_idx
     * @return
     */
    size_t add_connection_simple(int n1_idx, int n2_idx, size_t weight_idx);

    /**
     *
     * @param n1_idx
     * @param n2_idx
     * @param weight_idx
     * @param weight_value
     * @return
     */
    size_t add_connection_simple(int n1_idx, int n2_idx, size_t weight_idx, double weight_value);

    /**
     *
     * @param n1_idx
     * @param n2_idx
     * @param weight_idx
     * @param power_degree
     */
    void add_connection_shared_power1(int n1_idx, int n2_idx, size_t weight_idx);

    /**
     *
     * @param n1_idx
     * @param n2_idx
     * @param weight_idx
     * @param power_degree
     */
    void add_connection_shared_power2(int n1_idx, int n2_idx, size_t weight_idx);

    /**
     *
     * @param n1_idx
     * @param n2_idx
     * @param f
     * @param weight_indices
     * @param weight_values
     * @param n_weights
     */
    void add_connection_general(int n1_idx, int n2_idx, std::function<double(double *, size_t*, size_t)> *f,
                                size_t* weight_indices, double* weight_values, size_t n_weights);

    /**
     *
     * @param weight_ptr
     */
    void set_weight_array( std::vector<double>* weight_ptr );

    /**
     *
     * @param weights
     */
    void copy_weights(double *weights);

    /**
     *
     * @return
     */
    std::vector<double>* get_weights_ptr();

    /**
     *
     */
    void randomize_weights();

    /**
     *
     * @return
     */
    size_t get_n_inputs();

    /**
     *
     * @return
     */
    size_t get_n_outputs();

    /**
     *
     * @return
     */
    virtual size_t get_n_weights();

    /**
     *
     * @param input_neurons_indices
     */
    void specify_input_neurons(std::vector<size_t> &input_neurons_indices);

    /**
     *
     * @param output_neurons_indices
     */
    void specify_output_neurons(std::vector<size_t> &output_neurons_indices);

    /**
     *
     */
    void print_weights();

    /**
     *
     */
    void print_stats();
};


#endif //INC_4NEURO_NEURALNETWORK_H
