/**
 * DESCRIPTION OF THE CLASS
 *
 * @author Martin Beseda
 * @author Martin Mrovec
 * @author Michal Kravčenko
 * @date 2017 - 2018
 */

#ifndef INC_4NEURO_NEURONLINEAR_H
#define INC_4NEURO_NEURONLINEAR_H

#include "Neuron.h"

/**
 * Linear neuron class - uses activation function in the form f(x)=a*x + b,
 * 'x' being the neuron's potential
 */
class NeuronLinear:public Neuron, public IDifferentiable {
    friend class boost::serialization::access;

protected:
    template<class Archive>
    void serialize(Archive & ar, const unsigned int version){
        //TODO separate implementation to NeuronLinear.cpp!
        ar & boost::serialization::base_object<Neuron>(*this);
    };

public:

    Neuron* get_copy( );

    /**
     * Constructs the object of the Linear neuron with activation function
     * f(x) = b * x + a
     * @param[in] a First coefficient, stored in activation_function_parameters[0]
     * @param[in] b Second coefficient, stored in activation_function_parameters[1]
     */
    explicit NeuronLinear(double a = 0.0, double b = 0.0);

    /**
     * Evaluates 'b*x + a' and stores the result into the 'state' property
     */
    void activate( ) override;

    /**
     * Calculates the partial derivative of the activation function
     * f(x) = b*x + a at point x
     * @param[in] param_idx Index of the parameter to calculate derivative of
     * @return Partial derivative of the activation function according to the
     * 'param_idx'-th parameter. For 'param_idx'=0 returns 1, for 'param_idx=1'
     * return 'x' and otherwise returns 0.
     */
    double activation_function_eval_partial_derivative(int param_idx) override;

    /**
     * Calculates d/dx of (b*x + a) at point x
     * @return b
     */
    double activation_function_eval_derivative( ) override;

    /**
     * Returns a pointer to a Neuron with derivative as its activation function
     * @return
     */
    Neuron* get_derivative() override;

};


#endif //INC_4NEURO_NEURONLINEAR_H
