//
// Created by fluffymoo on 11.6.18.
//


#include "NeuronLogistic.h"

Neuron* NeuronLogistic_d2::get_copy( ){
    NeuronLogistic_d2* output = new NeuronLogistic_d2( this->activation_function_parameters[0],  this->activation_function_parameters[1]);

    return output;
}

NeuronLogistic_d2::NeuronLogistic_d2(double a, double b) {

    this->n_activation_function_parameters = 2;
    this->activation_function_parameters = new double[2];

    this->activation_function_parameters[0] = a;
    this->activation_function_parameters[1] = b;

    this->edges_in = new std::vector<Connection*>(0);
    this->edges_out = new std::vector<Connection*>(0);
}

void NeuronLogistic_d2::activate( ) {
    //[(1 + a) e^(b-x) (1 + e^(b - x))^(-1) - 1]a e^(b - x) (1 + e^(b - x))^(-1 - a)

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    double ex = std::pow(E, b - x);
    double ex2 = std::pow(ex + 1.0, -a - 1.0);
    double ex3 = 1.0 / (1.0 + ex);

    this->state = -a * ex * ex2 * (1.0 - (a + 1.0) * ex * ex3);

}

double NeuronLogistic_d2::activation_function_eval_partial_derivative(int param_idx ) {

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    if(param_idx == 0){
        //partial derivative according to parameter 'a'
        //(e^b (e^(b - x) + 1)^(-a) (a^2 (-e^b) log(e^(b - x) + 1) + a e^x log(e^(b - x) + 1) + 2 a e^b - e^x))/(e^b + e^x)^2
        double eb = std::pow(E, b);
        double ex = std::pow(E, x);
        double ebx= eb / ex;
        double ebxa = std::pow(ebx + 1.0, -a);
        double lbx = std::log(ebx + 1.0);

        return eb * ebxa * (a * lbx * ( a * (-eb) + ex ) + 2.0 * a * eb - ex) / ((eb + ex) * (eb + ex));
    }
    else if(param_idx == 1){
        //partial derivative according to parameter 'b'
        //-(a e^b (e^(b - x) + 1)^(-a) (a^2 e^(2 b) - 3 a e^(b + x) - e^(b + x) + e^(2 x)))/(e^b + e^x)^3

        double eb = std::pow(E, b);
        double ex = std::pow(E, x);
        double ebx= eb / ex;
        double ebxa = std::pow(ebx + 1.0, -a);

        return  - a * eb * ebxa * (a * a * eb * eb - 3.0 * a * ebx - ebx + ex * ex) / ((eb + ex) * (eb + ex) * (eb + ex));
    }

    return 0.0;
}

double NeuronLogistic_d2::activation_function_eval_derivative() {
    //(a e^b (1 + e^(b - x))^(-a) (a^2 e^(2 b) + e^(2 x) - e^(b + x) - 3 a e^(b + x)))/(e^b + e^x)^3
    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    double eb = std::pow(E, b);
    double ex = std::pow(E, x);
    double ebx= eb / ex;
    double ebxa = std::pow(ebx + 1.0, -a);

    return  a * eb * ebxa * (a * a * eb * eb - 3.0 * a * ebx - ebx + ex * ex) / ((eb + ex) * (eb + ex) * (eb + ex));
}


Neuron* NeuronLogistic_d1::get_copy( ){
    NeuronLogistic_d1* output = new NeuronLogistic_d1( this->activation_function_parameters[0],  this->activation_function_parameters[1]);

    return output;
}

NeuronLogistic_d1::NeuronLogistic_d1(double a, double b) {

    this->n_activation_function_parameters = 2;
    this->activation_function_parameters = new double[2];

    this->activation_function_parameters[0] = a;
    this->activation_function_parameters[1] = b;

    this->edges_in = new std::vector<Connection*>(0);
    this->edges_out = new std::vector<Connection*>(0);
}


void NeuronLogistic_d1::activate( ) {
    //a*e^(b - x)*(1 + e^(b - x))^(-1 - a)

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    double ex = std::pow(E, b - x);
    this->state = a * ex * std::pow(1.0 + ex, -a - 1.0);

}

double NeuronLogistic_d1::activation_function_eval_partial_derivative(int param_idx ) {

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    if(param_idx == 0){
        //partial derivative according to parameter 'a'
        //e^(b - x) (1 + e^(b - x))^(-1 - a)[1 - a log(1 + e^(b - x))]
        double ex = std::pow(E, b - x);
        double ex2= std::pow(1.0 + ex, -1.0 - a);

        return ex * ex2 * (1.0 - a * std::log(1.0 + ex));
    }
    else if(param_idx == 1){
        //partial derivative according to parameter 'b'
        //[(-1 - a) e^(b-x) (1 + e^(b - x))^(-1) + 1] * a e^(b - x) (1 + e^(b - x))^(-1 - a)
        double ex = std::pow(E, b - x);
        double ex2 = std::pow(ex + 1.0, -a - 1.0);
        double ex3 = 1.0 / (1.0 + ex);

        return a * ex * ex2 * (1.0 - (a + 1.0) * ex * ex3);
    }

    return 0.0;
}

double NeuronLogistic_d1::activation_function_eval_derivative() {
    //[(1 + a) e^(b-x) (1 + e^(b - x))^(-1) - 1]a e^(b - x) (1 + e^(b - x))^(-1 - a)

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    double ex = std::pow(E, b - x);
    double ex2 = std::pow(ex + 1.0, -a - 1.0);
    double ex3 = 1.0 / (1.0 + ex);

    return -a * ex * ex2 * (1.0 - (a + 1.0) * ex * ex3);
}

NeuronLogistic_d2* NeuronLogistic_d1::get_derivative() {

    //[(1 + a) e^(b-x) (1 + e^(b - x))^(-1) - 1]a e^(b - x) (1 + e^(b - x))^(-1 - a)
    NeuronLogistic_d2* output = nullptr;

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    output = new NeuronLogistic_d2(a, b);

    return output;

}


Neuron* NeuronLogistic::get_copy( ){
    NeuronLogistic* output = new NeuronLogistic( this->activation_function_parameters[0],  this->activation_function_parameters[1]);

    return output;
}

NeuronLogistic::NeuronLogistic(double a, double b) {

    this->n_activation_function_parameters = 2;
    this->activation_function_parameters = new double[2];

    this->activation_function_parameters[0] = a;
    this->activation_function_parameters[1] = b;

    this->edges_in = new std::vector<Connection*>(0);
    this->edges_out = new std::vector<Connection*>(0);
}

void NeuronLogistic::activate( ) {
    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    double ex = std::pow(E, b - x);
    this->state = std::pow(1.0 + ex, -a);

}

double NeuronLogistic::activation_function_eval_partial_derivative(int param_idx ) {

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    if(param_idx == 0){
        //partial derivative according to parameter 'a'
        double ex = std::pow(E, b - x);

        double exa= -std::pow(ex + 1.0, -a);

        return exa * std::log(ex + 1.0);
    }
    else if(param_idx == 1){
        //partial derivative according to parameter 'b'
        /**
         * TODO
         * Could be write as activation_function_get_derivative() * -1
         */
        double ex = std::pow(E, b - x);
        double ex2 = std::pow(ex + 1.0, -a - 1.0);

        return -a * ex * ex2;
    }

    return 0.0;

}


double NeuronLogistic::activation_function_eval_derivative( ) {

    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];
    double x = this->potential;

    double ex = std::pow(E, b - x);
    double ex2 = std::pow(ex + 1.0, -a - 1.0);

    return a * ex * ex2;

}

NeuronLogistic_d1* NeuronLogistic::get_derivative() {

    NeuronLogistic_d1 *output = nullptr;
    double a = this->activation_function_parameters[0];
    double b = this->activation_function_parameters[1];

    output = new NeuronLogistic_d1(a, b);
    output->set_potential( this->potential );

    return output;

}