/**
 * DESCRIPTION OF THE CLASS
 *
 * @author Martin Beseda
 * @author Martin Mrovec
 * @author Michal Kravčenko
 * @date 2017 - 2018
 */

#ifndef INC_4NEURO_NEURONTANH_H
#define INC_4NEURO_NEURONTANH_H

#include <cmath>
#include "Neuron.h"
#include "../constants.h"

class NeuronTanh:public Neuron, public IDifferentiable {
    friend class boost::serialization::access;

protected:
    template<class Archive>
    void serialize(Archive & ar, const unsigned int version){
        //TODO separate implementation to NeuronLogistic.cpp!
        ar & boost::serialization::base_object<Neuron>(*this);
    };

public:

    Neuron* get_copy( );

    /**
     * Constructs the object of the Hyperbolic Tangent >neuron with activation function
     * f(x) = (e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))
     * @param[in] a First coefficient, stored in activation_function_parameters[0]
     */
    explicit NeuronTanh(double a = 0.0);

    /**
     * Evaluates '(e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))' and stores the result into the 'state' property
     */
    void activate( ) override;

    /**
     * Calculates the partial derivative of the activation function
     * f(x) = (e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))
     * @param[in] param_idx Index of the parameter to calculate derivative of
     * @return Partial derivative of the activation function according to the
     * 'param_idx'-th parameter.
     */
    double activation_function_eval_partial_derivative(int param_idx) override;

    /**
     * TODO
     * Wrong return statement, there is no b
     * Calculates d/dx of (e^(x-a) - e^(a-x))/(e^(x-a) + e^(a-x))
     * @return a * e^(b - x) * [e^(b - x) + 1]^(-a)
     */
    double activation_function_eval_derivative( ) override;

    /**
     * Returns a pointer to a Neuron with derivative as its activation function
     * @return
     */
    Neuron* get_derivative() override;
};


#endif //INC_4NEURO_NEURONTANH_H
