/**
 * DESCRIPTION OF THE CLASS
 *
 * @author David Vojtek
 * @date 2018
 */

#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>
#include "../NetConnection/ConnectionWeight.h"

/**
 * Boost testing suite for testing ConnectionWeight.h
 */
BOOST_AUTO_TEST_SUITE(ConnectionWeight_test)

    /**
     * Test of construction of ConnectionWeight
     */
    BOOST_AUTO_TEST_CASE(ConnectionWeight_construction__test) {
     std::vector<double> * w_array = nullptr ;

     //Tests of no exception when, create new instance
     BOOST_CHECK_NO_THROW( ConnectionWeight conn(2, w_array ));
     BOOST_CHECK_NO_THROW( ConnectionWeight conn);
    }

    /**
     * Test of setParamIndices and SetParamIndex methods
     */
    BOOST_AUTO_TEST_CASE(ConnectionWeight_param_test){
        std::vector<double>  w_array= {0,1,2,3,4} ;
      
        ConnectionWeight conn(5, &w_array );
        ConnectionWeight connbad(7, &w_array);
        int para[5]={0,1,2,3,4};

        //Test of no exception when call SetParamIndices method on instance created with with correct parameters
        BOOST_CHECK_NO_THROW(conn.SetParamIndices(para));
        //Test of no exception when call SetParamIndex method on instance created with with correct parameters
        BOOST_CHECK_NO_THROW(conn.SetParamIndex(2,2));
        // Features not implemented yet
        //Test of expected exception out_of_range when call SetParamIndices method on instance created with  with incorrect parameters
        //BOOST_CHECK_THROW(connbad.SetParamIndices(para), std::out_of_range);
        //Test of expected exception out_of_range when call SetParamIndex method on instance created with incorrect parameters
        //BOOST_CHECK_THROW(connbad.SetParamIndex(2,8), std::out_of_range);
    }

    /**
     * Test of eval method
     */
    BOOST_AUTO_TEST_CASE(ConnectionWeight_eval__test) {
        std::vector<double>  w_array= {1,2,3,4,5} ;
        
        ConnectionWeight conn(5, &w_array );
        int para[5]={1,2,3,4,5};
        conn.SetParamIndices(para);

        //Test of correct output after calling eval() method
        BOOST_CHECK_EQUAL(120, conn.eval());
    }

    /**
     * Test of adjust_weights method
     */
    BOOST_AUTO_TEST_CASE(ConnectionWeight_weight_adjustment_test) {
        std::vector<double>  w_array= {1,2,3,4,5} ;
        double w_array2[5] = {5,4,3,2,1};
        
        ConnectionWeight conn(5, &w_array);
        int para[5]={0,1,2,3,4};
        conn.SetParamIndices(para);
        conn.adjust_weights(w_array2);

        //Test of correct eval output after adjusting weights
        BOOST_CHECK_EQUAL(7776, conn.eval());
    }

    /**
     * Test of set_weights method
     */
    BOOST_AUTO_TEST_CASE(ConnectionWeight_weight_set_test) {
        std::vector<double>  w_array= {2,3,4,5,6} ;
        double w_array2[5] = {1,2,3,4,5};
        ConnectionWeight conn(5, &w_array );
        int para[5]={0,1,2,3,4};
        conn.SetParamIndices(para);
        conn.set_weights(w_array2);

        //Test of correct eval output after setting new weights
        BOOST_CHECK_EQUAL(120, conn.eval());
    }


BOOST_AUTO_TEST_SUITE_END()