/**
 * DESCRIPTION OF THE CLASS
 *
 * @author David Vojtek
 * @date 2018
 */

#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>
#include "../Neuron/NeuronLinear.h"

/**
 * Boost testing suite for testing NeuronLinear.h
 * doesn't test inherited methods
 */
BOOST_AUTO_TEST_SUITE(neuronLinear_test)

    /**
     * Test of creating new instance of NeuronLinear
     */
    BOOST_AUTO_TEST_CASE(neuronLinear_construction__test) {
        NeuronLinear neuron(1.745, 784.4547);
        //Test of correct value of first activation function parameter
        BOOST_CHECK_EQUAL(neuron.activation_function_get_parameter(0), 1.745);
        //Test of correct value of second activation function parameter
        BOOST_CHECK_EQUAL(neuron.activation_function_get_parameter(1), 784.4547);
    }

    /**
     * Test of activate method
     */
    BOOST_AUTO_TEST_CASE(neuronLinear_activate__test) {
        NeuronLinear neuron(5.0, 1.0);
        neuron.activate();
        //Test of correct state after activate neuron
        BOOST_CHECK_EQUAL(5.0, neuron.get_state());
    }

    /**
     * Test of derivative methods
     */
    BOOST_AUTO_TEST_CASE(neuronLinear_derivative_test) {
        NeuronLinear neuron(5.0, 3.0);
        
        //Test of correct output of activation_function_get_derivative method
        BOOST_CHECK_EQUAL(3.0, neuron.activation_function_get_derivative());
        //Tests of correct outputs of activation_function_get_partial_derivative method
        BOOST_CHECK_EQUAL(1.0, neuron.activation_function_get_partial_derivative(0));
        BOOST_CHECK_EQUAL(0.0, neuron.activation_function_get_partial_derivative(1));
        BOOST_CHECK_EQUAL(0.0, neuron.activation_function_get_partial_derivative(10000));
    }

BOOST_AUTO_TEST_SUITE_END()
