/**
 * DESCRIPTION OF THE CLASS
 *
 * @author David Vojtek
 * @date 2018
 */

#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>

#include "../Neuron/NeuronTanh.h"
/**
 * Boost testing suite for testing NeuronTanh.h
 * doesn't test inherited methods
 */
BOOST_AUTO_TEST_SUITE(neuronTanh_test)

    /**
     * Test of creating new instance of NeuronTanh
     */
    BOOST_AUTO_TEST_CASE(neuronTanh_construction__test) {
        NeuronTanh neuron(1.745);
        //Test of correct value of activation function parameter
        BOOST_CHECK_EQUAL(neuron.activation_function_get_parameter(0), 1.745);
    }

    /**
     * Test of activate method
     */
    BOOST_AUTO_TEST_CASE(neuronTanh_activate__test) {
        NeuronTanh neuron(2.0);
        neuron.activate();
        //Test of correct state after activate neuron
        BOOST_CHECK_CLOSE(-0.96402758007581, neuron.get_state(), 0.00001);
    }

    /**
    * Test of derivative methods
    */
    BOOST_AUTO_TEST_CASE(neuronTanh_derivative_test) {
        NeuronTanh neuron(2.0);

        //Test of correct output of activation_function_get_derivative method
        BOOST_CHECK_CLOSE(-0.0706508248531644, neuron.activation_function_get_derivative(), 0.00001);
        //Tests of correct outputs of activation_function_get_partial_derivative method
        BOOST_CHECK_CLOSE(-0.0706508248531644, neuron.activation_function_get_partial_derivative(0), 0.00001);
        BOOST_CHECK_EQUAL(0.0, neuron.activation_function_get_partial_derivative(10000));
    }

BOOST_AUTO_TEST_SUITE_END()